@@ -3,9 +3,11 @@
 
     This file is part of libkleopatra, the KDE keymanagement library
     SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
+    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     DN parsing:
-    SPDX-FileCopyrightText: 2002 g 10 Code GmbH
+    SPDX-FileCopyrightText: 2002 g10 Code GmbH
     SPDX-FileCopyrightText: 2004 Klarälvdalens Datakonsult AB
 
     SPDX-License-Identifier: GPL-2.0-or-later
@@ -15,27 +17,54 @@
 
 #include "oidmap.h"
 
-#include "ui/dnattributeorderconfigwidget.h"
+#include <KLazyLocalizedString>
 
-#include <KConfig>
-#include <KLocalizedString>
-
-
-#include <iostream>
-#include <iterator>
 #include <algorithm>
-#include <map>
-
-#include <string.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <KConfigGroup>
-#include <KSharedConfig>
 
 #ifdef _MSC_VER
+#include <string.h>
 #define strcasecmp _stricmp
 #endif
 
+namespace {
+static const QStringList defaultOrder = {
+    QStringLiteral("CN"),
+    QStringLiteral("L"),
+    QStringLiteral("_X_"),
+    QStringLiteral("OU"),
+    QStringLiteral("O"),
+    QStringLiteral("C"),
+};
+
+class DNAttributeOrderStore
+{
+    DNAttributeOrderStore()
+        : mAttributeOrder{defaultOrder}
+    {
+    }
+
+public:
+    static DNAttributeOrderStore *instance()
+    {
+        static DNAttributeOrderStore *self = new DNAttributeOrderStore();
+        return self;
+    }
+
+    const QStringList &attributeOrder() const
+    {
+        return mAttributeOrder.empty() ? defaultOrder : mAttributeOrder;
+    }
+
+    void setAttributeOrder(const QStringList &order)
+    {
+        mAttributeOrder = order;
+    }
+
+private:
+    QStringList mAttributeOrder;
+};
+}
+
 class Kleo::DN::Private
 {
 public:
@@ -95,7 +124,8 @@ struct DnPair {
 static char *
 trim_trailing_spaces(char *string)
 {
-    char *p, *mark;
+    char *p;
+    char *mark;
 
     for (mark = nullptr, p = string; *p; p++) {
         if (isspace(*p)) {
@@ -119,13 +149,15 @@ trim_trailing_spaces(char *string)
 static const unsigned char *
 parse_dn_part(DnPair *array, const unsigned char *string)
 {
-    const unsigned char *s, *s1;
+    const unsigned char *s;
+    const unsigned char *s1;
     size_t n;
     char *p;
 
     /* parse attributeType */
-    for (s = string + 1; *s && *s != '='; s++)
+    for (s = string + 1; *s && *s != '='; s++) {
         ;
+    }
     if (!*s) {
         return nullptr;    /* error */
     }
@@ -139,12 +171,13 @@ parse_dn_part(DnPair *array, const unsigned char *string)
     p[n] = 0;
     trim_trailing_spaces((char *)p);
     // map OIDs to their names:
-    for (unsigned int i = 0; i < numOidMaps; ++i)
+    for (unsigned int i = 0; i < numOidMaps; ++i) {
         if (!strcasecmp((char *)p, oidmap[i].oid)) {
             free(p);
             p = strdup(oidmap[i].name);
             break;
         }
+    }
     array->key = p;
     string = s + 1;
 
@@ -235,9 +268,10 @@ parse_dn(const unsigned char *string)
         if (!string) {
             goto failure;
         }
-        if (pair.key && pair.value)
+        if (pair.key && pair.value) {
             result.push_back(Kleo::DN::Attribute(QString::fromUtf8(pair.key),
                                                  QString::fromUtf8(pair.value)));
+        }
         free(pair.key);
         free(pair.value);
 
@@ -290,17 +324,18 @@ static QString
 serialise(const QVector<Kleo::DN::Attribute> &dn, const QString &sep)
 {
     QStringList result;
-    for (QVector<Kleo::DN::Attribute>::const_iterator it = dn.begin(); it != dn.end(); ++it)
+    for (QVector<Kleo::DN::Attribute>::const_iterator it = dn.begin(); it != dn.end(); ++it) {
         if (!(*it).name().isEmpty() && !(*it).value().isEmpty()) {
             result.push_back((*it).name().trimmed() + QLatin1Char('=') + dn_escape((*it).value().trimmed()));
         }
+    }
     return result.join(sep);
 }
 
 static Kleo::DN::Attribute::List
 reorder_dn(const Kleo::DN::Attribute::List &dn)
 {
-    const QStringList &attrOrder = Kleo::DNAttributeMapper::instance()->attributeOrder();
+    const QStringList &attrOrder = Kleo::DN::attributeOrder();
 
     Kleo::DN::Attribute::List unknownEntries;
     Kleo::DN::Attribute::List result;
@@ -308,24 +343,27 @@ reorder_dn(const Kleo::DN::Attribute::List &dn)
     result.reserve(dn.size());
 
     // find all unknown entries in their order of appearance
-    for (Kleo::DN::const_iterator it = dn.begin(); it != dn.end(); ++it)
+    for (Kleo::DN::const_iterator it = dn.begin(); it != dn.end(); ++it) {
         if (!attrOrder.contains((*it).name())) {
             unknownEntries.push_back(*it);
         }
+    }
 
     // process the known attrs in the desired order
-    for (QStringList::const_iterator oit = attrOrder.begin(); oit != attrOrder.end(); ++oit)
+    for (QStringList::const_iterator oit = attrOrder.begin(); oit != attrOrder.end(); ++oit) {
         if (*oit == QLatin1String("_X_")) {
             // insert the unknown attrs
             std::copy(unknownEntries.begin(), unknownEntries.end(),
                       std::back_inserter(result));
             unknownEntries.clear(); // don't produce dup's
         } else {
-            for (Kleo::DN::const_iterator dnit = dn.begin(); dnit != dn.end(); ++dnit)
+            for (Kleo::DN::const_iterator dnit = dn.begin(); dnit != dn.end(); ++dnit) {
                 if ((*dnit).name() == *oit) {
                     result.push_back(*dnit);
                 }
+            }
         }
+    }
 
     return result;
 }
@@ -391,6 +429,24 @@ const Kleo::DN &Kleo::DN::operator=(const DN &that)
     return *this;
 }
 
+// static
+QStringList Kleo::DN::attributeOrder()
+{
+    return DNAttributeOrderStore::instance()->attributeOrder();
+}
+
+// static
+void Kleo::DN::setAttributeOrder(const QStringList &order)
+{
+    DNAttributeOrderStore::instance()->setAttributeOrder(order);
+}
+
+// static
+QStringList Kleo::DN::defaultAttributeOrder()
+{
+    return defaultOrder;
+}
+
 QString Kleo::DN::prettyDN() const
 {
     if (!d) {
@@ -444,11 +500,11 @@ QString Kleo::DN::operator[](const QString &attr) const
         return QString();
     }
     const QString attrUpper = attr.toUpper();
-    for (QVector<Attribute>::const_iterator it = d->attributes.constBegin();
-            it != d->attributes.constEnd(); ++it)
+    for (QVector<Attribute>::const_iterator it = d->attributes.constBegin(); it != d->attributes.constEnd(); ++it) {
         if ((*it).name() == attrUpper) {
             return (*it).value();
         }
+    }
     return QString();
 }
 
@@ -468,116 +524,37 @@ Kleo::DN::const_iterator Kleo::DN::end() const
 
 namespace
 {
-struct ltstr {
-    bool operator()(const char *s1, const char *s2) const
-    {
-        return qstrcmp(s1, s2) < 0;
-    }
+static const QMap<QString, KLazyLocalizedString> attributeNamesAndLabels = {
+    { QStringLiteral("CN"), kli18n("Common name") },
+    { QStringLiteral("SN"), kli18n("Surname") },
+    { QStringLiteral("GN"), kli18n("Given name") },
+    { QStringLiteral("L"), kli18n("Location") },
+    { QStringLiteral("T"), kli18n("Title") },
+    { QStringLiteral("OU"), kli18n("Organizational unit") },
+    { QStringLiteral("O"), kli18n("Organization") },
+    { QStringLiteral("PC"), kli18n("Postal code") },
+    { QStringLiteral("C"), kli18n("Country code") },
+    { QStringLiteral("SP"), kli18n("State or province") },
+    { QStringLiteral("DC"), kli18n("Domain component") },
+    { QStringLiteral("BC"), kli18n("Business category") },
+    { QStringLiteral("EMAIL"), kli18n("Email address") },
+    { QStringLiteral("MAIL"), kli18n("Mail address") },
+    { QStringLiteral("MOBILE"), kli18n("Mobile phone number") },
+    { QStringLiteral("TEL"), kli18n("Telephone number") },
+    { QStringLiteral("FAX"), kli18n("Fax number") },
+    { QStringLiteral("STREET"), kli18n("Street address") },
+    { QStringLiteral("UID"), kli18n("Unique ID") }
 };
 }
 
-static const QStringList defaultOrder = {
-    QStringLiteral("CN"),
-    QStringLiteral("L"),
-    QStringLiteral("_X_"),
-    QStringLiteral("OU"),
-    QStringLiteral("O"),
-    QStringLiteral("C"),
-};
-
-static std::pair<const char *, const char *> const attributeLabels[] = {
-#define MAKE_PAIR(x,y) std::pair<const char*,const char*>( x, y )
-    MAKE_PAIR("CN", I18N_NOOP("Common name")),
-    MAKE_PAIR("SN", I18N_NOOP("Surname")),
-    MAKE_PAIR("GN", I18N_NOOP("Given name")),
-    MAKE_PAIR("L",  I18N_NOOP("Location")),
-    MAKE_PAIR("T",  I18N_NOOP("Title")),
-    MAKE_PAIR("OU", I18N_NOOP("Organizational unit")),
-    MAKE_PAIR("O",  I18N_NOOP("Organization")),
-    MAKE_PAIR("PC", I18N_NOOP("Postal code")),
-    MAKE_PAIR("C",  I18N_NOOP("Country code")),
-    MAKE_PAIR("SP", I18N_NOOP("State or province")),
-    MAKE_PAIR("DC", I18N_NOOP("Domain component")),
-    MAKE_PAIR("BC", I18N_NOOP("Business category")),
-    MAKE_PAIR("EMAIL", I18N_NOOP("Email address")),
-    MAKE_PAIR("MAIL", I18N_NOOP("Mail address")),
-    MAKE_PAIR("MOBILE", I18N_NOOP("Mobile phone number")),
-    MAKE_PAIR("TEL", I18N_NOOP("Telephone number")),
-    MAKE_PAIR("FAX", I18N_NOOP("Fax number")),
-    MAKE_PAIR("STREET", I18N_NOOP("Street address")),
-    MAKE_PAIR("UID", I18N_NOOP("Unique ID"))
-#undef MAKE_PAIR
-};
-static const unsigned int numAttributeLabels = sizeof attributeLabels / sizeof * attributeLabels;
-
-class Kleo::DNAttributeMapper::Private
+// static
+QStringList Kleo::DN::attributeNames()
 {
-public:
-    Private();
-    std::map<const char *, const char *, ltstr> map;
-    QStringList attributeOrder;
-};
-
-Kleo::DNAttributeMapper::Private::Private()
-    : map(attributeLabels, attributeLabels + numAttributeLabels) {}
-
-Kleo::DNAttributeMapper::DNAttributeMapper()
-{
-    d = new Private();
-    const KConfigGroup config(KSharedConfig::openConfig(), "DN");
-    d->attributeOrder = config.readEntry("AttributeOrder", defaultOrder);
-    mSelf = this;
+    return attributeNamesAndLabels.keys();
 }
 
-Kleo::DNAttributeMapper::~DNAttributeMapper()
+// static
+QString Kleo::DN::attributeNameToLabel(const QString &name)
 {
-    mSelf = nullptr;
-    delete d; d = nullptr;
-}
-
-Kleo::DNAttributeMapper *Kleo::DNAttributeMapper::mSelf = nullptr;
-
-const Kleo::DNAttributeMapper *Kleo::DNAttributeMapper::instance()
-{
-    if (!mSelf) {
-        (void)new DNAttributeMapper();
-    }
-    return mSelf;
-}
-
-QString Kleo::DNAttributeMapper::name2label(const QString &s) const
-{
-    const std::map<const char *, const char *, ltstr>::const_iterator it
-        = d->map.find(s.trimmed().toUpper().toLatin1().constData());
-    if (it == d->map.end()) {
-        return QString();
-    }
-    return i18n(it->second);
-}
-
-QStringList Kleo::DNAttributeMapper::names() const
-{
-    QStringList result;
-    for (std::map<const char *, const char *, ltstr>::const_iterator it = d->map.begin(); it != d->map.end(); ++it) {
-        result.push_back(QLatin1String(it->first));
-    }
-    return result;
-}
-
-const QStringList &Kleo::DNAttributeMapper::attributeOrder() const
-{
-    return d->attributeOrder;
-}
-
-void Kleo::DNAttributeMapper::setAttributeOrder(const QStringList &order)
-{
-    d->attributeOrder = order.empty() ? defaultOrder : order;
-
-    KConfigGroup config(KSharedConfig::openConfig(), "DN");
-    config.writeEntry("AttributeOrder", order);
-}
-
-Kleo::DNAttributeOrderConfigWidget *Kleo::DNAttributeMapper::configWidget(QWidget *parent) const
-{
-    return new DNAttributeOrderConfigWidget(mSelf, parent);
+    return attributeNamesAndLabels.value(name.trimmed().toUpper()).toString();
 }
