@@ -14,6 +14,7 @@
 #include <functional>
 
 #include <KLocalizedString>
+#include <KLazyLocalizedString>
 
 #include <gpgme++/key.h>
 #include <gpgme++/tofuinfo.h>
@@ -23,37 +24,37 @@
 
 static const struct {
     Kleo::CryptoMessageFormat format;
-    const char *displayName;
+    const KLazyLocalizedString displayName;
     const char *configName;
 } cryptoMessageFormats[] = {
     {
         Kleo::InlineOpenPGPFormat,
-        I18N_NOOP("Inline OpenPGP (deprecated)"),
+        kli18n("Inline OpenPGP (deprecated)"),
         "inline openpgp"
     },
     {
         Kleo::OpenPGPMIMEFormat,
-        I18N_NOOP("OpenPGP/MIME"),
+        kli18n("OpenPGP/MIME"),
         "openpgp/mime"
     },
     {
         Kleo::SMIMEFormat,
-        I18N_NOOP("S/MIME"),
+        kli18n("S/MIME"),
         "s/mime"
     },
     {
         Kleo::SMIMEOpaqueFormat,
-        I18N_NOOP("S/MIME Opaque"),
+        kli18n("S/MIME Opaque"),
         "s/mime opaque"
     },
     {
         Kleo::AnySMIME,
-        I18N_NOOP("Any S/MIME"),
+        kli18n("Any S/MIME"),
         "any s/mime"
     },
     {
         Kleo::AnyOpenPGP,
-        I18N_NOOP("Any OpenPGP"),
+        kli18n("Any OpenPGP"),
         "any openpgp"
     },
 };
@@ -65,20 +66,22 @@ const char *Kleo::cryptoMessageFormatToString(Kleo::CryptoMessageFormat f)
     if (f == AutoFormat) {
         return "auto";
     }
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f == cryptoMessageFormats[i].format) {
             return cryptoMessageFormats[i].configName;
         }
+    }
     return nullptr;
 }
 
 QStringList Kleo::cryptoMessageFormatsToStringList(unsigned int f)
 {
     QStringList result;
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f & cryptoMessageFormats[i].format) {
             result.push_back(QLatin1String(cryptoMessageFormats[i].configName));
         }
+    }
     return result;
 }
 
@@ -87,20 +90,22 @@ QString Kleo::cryptoMessageFormatToLabel(Kleo::CryptoMessageFormat f)
     if (f == AutoFormat) {
         return i18n("Any");
     }
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (f == cryptoMessageFormats[i].format) {
-            return i18n(cryptoMessageFormats[i].displayName);
+            return KLocalizedString(cryptoMessageFormats[i].displayName).toString();
         }
+    }
     return QString();
 }
 
 Kleo::CryptoMessageFormat Kleo::stringToCryptoMessageFormat(const QString &s)
 {
     const QString t = s.toLower();
-    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i)
+    for (unsigned int i = 0; i < numCryptoMessageFormats; ++i) {
         if (t == QLatin1String(cryptoMessageFormats[i].configName)) {
             return cryptoMessageFormats[i].format;
         }
+    }
     return AutoFormat;
 }
 
@@ -247,7 +252,7 @@ namespace {
 
 bool hasTrustedSignature(const GpgME::UserID &uid)
 {
-    // lazily intialized cache
+    // lazily initialized cache
     static std::shared_ptr<const Kleo::KeyCache> keyCache;
     if (!keyCache) {
         keyCache = Kleo::KeyCache::instance();
