@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-only
 */
 
-#ifndef __KLEO_TEST_KEYLISTER_H__
-#define __KLEO_TEST_KEYLISTER_H__
+#pragma once
 
 #include "ui/keylistview.h"
 
@@ -23,11 +22,10 @@ class CertListView : public Kleo::KeyListView
     Q_OBJECT
 public:
     explicit CertListView(QWidget *parent = nullptr, Qt::WindowFlags f = {});
-    ~CertListView();
+    ~CertListView() override;
 
 public Q_SLOTS:
     void slotResult(const GpgME::KeyListResult &result);
     void slotStart();
 };
 
-#endif // __KLEO_TEST_KEYLISTER_H__
