@@ -7,14 +7,17 @@
     SPDX-FileCopyrightText: 2016 Bundesamt für Sicherheit in der Informationstechnik
     SPDX-FileContributor: Intevation GmbH
 
-    SPDX-FileCopyrightText: 2020 g10 Code GmbH
+    SPDX-FileCopyrightText: 2020-2022 g10 Code GmbH
+    SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
 #include "gnupg.h"
 
+#include "utils/assuan.h"
 #include "utils/compat.h"
+#include "utils/cryptoconfig.h"
 #include "utils/hex.h"
 
 #include <gpgme++/engineinfo.h>
@@ -26,6 +29,7 @@
 
 #include "libkleo_debug.h"
 
+#include <QDateTime>
 #include <QDir>
 #include <QFile>
 #include <QString>
@@ -33,6 +37,8 @@
 #include <QByteArray>
 #include <QStandardPaths>
 #include <QCoreApplication>
+#include <QRegularExpression>
+#include <QPointer>
 #include <gpg-error.h>
 
 #ifdef Q_OS_WIN
@@ -212,6 +218,26 @@ bool Kleo::gpg4winSignedversion()
     return Gpg4win::instance()->isSignedVersion();
 }
 
+QString Kleo::gpg4winVersionNumber()
+{
+    // extract the actual version number from the string returned by Gpg4win::version();
+    // we assume that Gpg4win::version() returns a version number (conforming to the semantic
+    // versioning spec) optionally prefixed with some text followed by a dash,
+    // e.g. "Gpg4win-3.1.15-beta15"; see https://dev.gnupg.org/T5663
+    static const QRegularExpression catchSemVerRegExp{
+        QLatin1String{R"(-([0-9]+(?:\.[0-9]+)*(?:-[.0-9A-Za-z-]+)?(?:\+[.0-9a-zA-Z-]+)?)$)"}};
+
+    QString ret;
+    const auto match = catchSemVerRegExp.match(gpg4winVersion());
+    if (match.hasMatch()) {
+        ret = match.captured(1);
+    } else {
+        ret = gpg4winVersion();
+    }
+    qCDebug(LIBKLEO_LOG) << __func__ << "returns" << ret;
+    return ret;
+}
+
 QString Kleo::gpg4winVersion()
 {
     return Gpg4win::instance()->version();
@@ -294,7 +320,7 @@ QString Kleo::gpgConfListDir(const char *which)
         return QString();
     }
     const QList<QByteArray> lines = gpgConf.readAllStandardOutput().split('\n');
-    for (const QByteArray &line : lines)
+    for (const QByteArray &line : lines) {
         if (line.startsWith(which) && line[qstrlen(which)] == ':') {
             const int begin = qstrlen(which) + 1;
             int end = line.size();
@@ -306,6 +332,7 @@ QString Kleo::gpgConfListDir(const char *which)
                                    << " for '" << which << "'entry";
             return result;
         }
+    }
     qCDebug(LIBKLEO_LOG) << "gpgConfListDir(): didn't find '" << which << "'"
                            << "entry in output:\n" << gpgConf.readAllStandardError().constData();
     return QString();
@@ -417,12 +444,23 @@ bool Kleo::haveKeyserverConfigured()
         // since 2.1.19 there is a builtin keyserver
         return true;
     }
-    const QGpgME::CryptoConfig *const config = QGpgME::cryptoConfig();
-    if (!config) {
-        return false;
+    return !Kleo::keyserver().isEmpty();
+}
+
+QString Kleo::keyserver()
+{
+    QString result = getCryptoConfigStringValue("gpg", "keyserver");
+    if (result.isEmpty()) {
+        result = getCryptoConfigStringValue("dirmngr", "keyserver");
     }
-    const QGpgME::CryptoConfigEntry *const entry = Kleo::getCryptoConfigEntry(config, "gpg", "keyserver");
-    return entry && !entry->stringValue().isEmpty();
+    return result;
+}
+
+bool Kleo::haveX509DirectoryServerConfigured()
+{
+    return !getCryptoConfigUrlList("dirmngr", "ldapserver").empty()
+        || !getCryptoConfigUrlList("dirmngr", "LDAP Server").empty()
+        || !getCryptoConfigUrlList("gpgsm", "keyserver").empty();
 }
 
 bool Kleo::gpgComplianceP(const char *mode)
@@ -432,6 +470,28 @@ bool Kleo::gpgComplianceP(const char *mode)
     return entry && entry->stringValue() == QString::fromLatin1(mode);
 }
 
+bool Kleo::gnupgUsesDeVsCompliance()
+{
+    return getCryptoConfigStringValue("gpg", "compliance") == QLatin1String{"de-vs"};
+}
+
+bool Kleo::gnupgIsDeVsCompliant()
+{
+    if (!gnupgUsesDeVsCompliance()) {
+        return false;
+    }
+    // The pseudo option compliance_de_vs was fully added in 2.2.34;
+    // For versions between 2.2.28 and 2.2.33 there was a broken config
+    // value with a wrong type. So for them we add an extra check. This
+    // can be removed in future versions because. For GnuPG we could assume
+    // non-compliance for older versions as versions of Kleopatra for
+    // which this matters are bundled with new enough versions of GnuPG anyway
+    if (engineIsVersion(2, 2, 28) && !engineIsVersion(2, 2, 34)) {
+        return true;
+    }
+    return getCryptoConfigIntValue("gpg", "compliance_de_vs", 0) != 0;
+}
+
 enum GpgME::UserID::Validity Kleo::keyValidity(const GpgME::Key &key)
 {
     enum UserID::Validity validity = UserID::Validity::Unknown;
@@ -496,3 +556,131 @@ QString Kleo::stringFromGpgOutput(const QByteArray &ba)
     return QString::fromLocal8Bit(ba);
 #endif
 }
+
+QStringList Kleo::backendVersionInfo()
+{
+    QStringList versions;
+    if (Kleo::engineIsVersion(2, 2, 24, GpgME::GpgConfEngine)) {
+        QProcess p;
+        qCDebug(LIBKLEO_LOG) << "Running gpgconf --show-versions ...";
+        p.start(Kleo::gpgConfPath(), {QStringLiteral("--show-versions")});
+        // wait at most 1 second
+        if (!p.waitForFinished(1000)) {
+            qCDebug(LIBKLEO_LOG) << "Running gpgconf --show-versions timed out after 1 second.";
+        } else if (p.exitStatus() != QProcess::NormalExit || p.exitCode() != 0) {
+            qCDebug(LIBKLEO_LOG) << "Running gpgconf --show-versions failed:" << p.errorString();
+            qCDebug(LIBKLEO_LOG) << "gpgconf stderr:" << p.readAllStandardError();
+            qCDebug(LIBKLEO_LOG) << "gpgconf stdout:" << p.readAllStandardOutput();
+        } else {
+            const QByteArray output = p.readAllStandardOutput().replace("\r\n", "\n");
+            qCDebug(LIBKLEO_LOG) << "gpgconf stdout:" << p.readAllStandardOutput();
+            const auto lines = output.split('\n');
+            for (const auto &line : lines) {
+                if (line.startsWith("* GnuPG") || line.startsWith("* Libgcrypt")) {
+                    const auto components = line.split(' ');
+                    versions.push_back(QString::fromLatin1(components.at(1) + ' ' + components.value(2)));
+                }
+            }
+        }
+    }
+    return versions;
+}
+
+namespace
+{
+
+template <typename Function1, typename Function2>
+auto startGpgConf(const QStringList &arguments, Function1 onSuccess, Function2 onFailure)
+{
+    auto process = new QProcess;
+    process->setProgram(Kleo::gpgConfPath());
+    process->setArguments(arguments);
+
+    QObject::connect(process, &QProcess::started,
+                     [process]() {
+                         qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") was started successfully";
+                     });
+    QObject::connect(process, &QProcess::errorOccurred,
+                     [process, onFailure](auto error) {
+                         qCDebug(LIBKLEO_LOG).nospace() << "Error while running gpgconf (" << process << "): " << error;
+                         process->deleteLater();
+                         onFailure();
+                     });
+    QObject::connect(process, &QProcess::readyReadStandardError,
+                     [process]() {
+                         for (const auto &line : process->readAllStandardError().trimmed().split('\n')) {
+                            qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") stderr: " << line;
+                         }
+                     });
+    QObject::connect(process, &QProcess::readyReadStandardOutput,
+                     [process]() { (void)process->readAllStandardOutput(); /* ignore stdout */ });
+    QObject::connect(process, qOverload<int, QProcess::ExitStatus>(&QProcess::finished),
+                     [process, onSuccess, onFailure](int exitCode, QProcess::ExitStatus exitStatus) {
+                         if (exitStatus == QProcess::NormalExit) {
+                            qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") exited (exit code: " << exitCode << ")";
+                            if (exitCode == 0) {
+                                onSuccess();
+                            } else {
+                                onFailure();
+                            }
+                         } else {
+                            qCDebug(LIBKLEO_LOG).nospace() << "gpgconf (" << process << ") crashed (exit code: " << exitCode << ")";
+                            onFailure();
+                         }
+                         process->deleteLater();
+                     });
+
+    qCDebug(LIBKLEO_LOG).nospace() << "Starting gpgconf (" << process << ") with arguments " << process->arguments().join(QLatin1Char(' ')) << " ...";
+    process->start();
+
+    return process;
+}
+
+static auto startGpgConf(const QStringList &arguments)
+{
+    return startGpgConf(arguments, [](){}, [](){});
+}
+
+}
+
+void Kleo::launchGpgAgent()
+{
+    static QPointer<QProcess> process;
+    static qint64 mSecsSinceEpochOfLastLaunch = 0;
+    static int numberOfFailedLaunches = 0;
+
+    if (Kleo::Assuan::agentIsRunning()) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": gpg-agent is already running";
+        return;
+    }
+    if (process) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": gpg-agent is already being launched";
+        return;
+    }
+    const auto now = QDateTime::currentMSecsSinceEpoch();
+    if (now - mSecsSinceEpochOfLastLaunch < 1000) {
+        // reduce attempts to launch the agent to 1 attempt per second
+        return;
+    }
+    mSecsSinceEpochOfLastLaunch = now;
+    if (numberOfFailedLaunches > 5) {
+        qCWarning(LIBKLEO_LOG) << __func__ << ": Launching gpg-agent failed" << numberOfFailedLaunches << "times in a row. Giving up.";
+        return;
+    }
+
+    process = startGpgConf({QStringLiteral("--launch"), QStringLiteral("gpg-agent")},
+                           []() { numberOfFailedLaunches = 0; },
+                           []() { numberOfFailedLaunches++; });
+}
+
+void Kleo::killDaemons()
+{
+    static QPointer<QProcess> process;
+
+    if (process) {
+        qCDebug(LIBKLEO_LOG) << __func__ << ": The daemons are already being shut down";
+        return;
+    }
+
+    process = startGpgConf({QStringLiteral("--kill"), QStringLiteral("all")});
+}
